# PARALLAX

jQuery **Parallax** plugin.

* Simply target all devices with 'responsive mode'.
* SASS/SCSS files for simple and quick UI customisations.
* Only 2KB (minified).
* Fully compatible with **t** css-framework
* Runs in strict mode.


## Dependencies

**jQuery**

Developed and tested with jQuery 3.2.1.

## Settings / Options

Name | Default | Type | Description | Required or optional
-----|---------|------|-------------|----------------------
parallaxEmphasis | 50 | int (>= 0, < 100) | Defines the maximum top and bottom offset for the parallax (percentage value of the image height). | optional
reverseParallax | false | bool | Invert the parallax direction. | optional
responsive | false | false or array of objects| Enable responsive mode by passing an array of object with settings for different breakpoints. For detailed infos check the section **'responsive mode'**. | optional

## Usage

1. Include the JS script **parallax.js** - or the minified production script **parallax.min.js** in the head or the body of your HTML file.
2. Include the CSS file  **parallax.css** in the head of your HTML file or include the SCSS files in your project. Adapt the CSS rules to match your website's design.
3. Initialise the widget within an inline script tag, or in an external JS file.


### HTML

Use following HTML markup:

1) Base markup for images with semantic meaning:

```
<figure class="parallax" id="parallax">
  <img src="parallax-img.jpg" alt="alt text for image" class="parallax__img" />
  <figcaption class="parallax__figcaption">Caption for figure</figcaption>
</figure>
```

2) Base markup for decorative images:

```javascript
<div class="parallax" id="parallax">
  <img src="parallax-img.jpg" alt="" class="parallax__img" role="presentation"/>
  <span class="parallax__figcaption">Text</span>
</div>
```

### Globally change default options

Globally overwrite the default settings as follows:

```javascript
 $.fn.parallax.defaultSettings = {
  option1: value,
  option2: value
 };

```


### JS: Initialisation

Initialise the plugin as follows:

```javascript
$('#parallax').parallax({
  option: value
});
```

## Responsive mode

The plugin supports an **advanced responsive mode**. This mode gives authors the possibility to change the settings dynamically based on viewport width. The responsive mode is enabled through the option **'responsive'**. This option **accepts an array of objects** with the options for differen screen widths (see code below). 

```javascript
$('#parallax').parallax({
    responsive: [
      {
        breakpoint: 1,
        parallaxEmphasis: 0
      },
      {
        breakpoint: 768,
        parallaxEmphasis: 40,
        reverseParallax: true
      },
      {
        breakpoint: 992,
        parallaxEmphasis: 60
      }
    ]
```
The option **'breakpoint'** sets the minimum screen width (in px) and is **allowed only in responsive mode**, e.g. as a property of an object of the 'responsive' array. Except for 'breakpoint' and 'responsive', all other options are not restricted to a specific mode.

Options that applies to any screen width **can and should** be set outside of the option **'responsive'** to keep code clear and reduce the amount of operations needed for the initialisation of the plugin.
Consider following example:


```javascript
$('#parallax').parallax({
    responsive: [
      {
        breakpoint: 1,
        parallaxEmphasis: 30
        [...]
      },
      {
        breakpoint: 768,
        parallaxEmphasis: 30
        [...]
      },
      {
        breakpoint: 992,
        parallaxEmphasis: 30
        [...]
      }
    ]
  });
```

Although this code would perfectly work, it is strongly recommended to set the option **'parallaxEmphasis'** only once:

```javascript
$('#parallax').parallax({
    parallaxEmphasis: 30
    responsive: [
      {
        breakpoint: 1,
        [...]
      },
      {
        breakpoint: 768,
        [...]
      }, 
      {
        breakpoint: 992,
        [...]
      }
    ]
  });
```

When using responsive mode, it is important to **cover all screen widths** by setting the first **'breakpoint'** to **1** or to the minimum screen width possible. Also, for the plugin to work correctly, all objects **must be sorted ascendingly by breakpoint**:

```javascript
$('#parallax').parallax({
    responsive: [
      {
        breakpoint: 768,
        [...]
      },
      {
        breakpoint: 1,
        [...]
      },
      {
        breakpoint: 992,
        [...]
      }
    ]
  });
```

This code would cause an error and the plugin wont work correctly!

The correct initialisation code is instead the following:

```javascript
$('#parallax').parallax({
    responsive: [
      {
        breakpoint: 1,    //first breakpoint
        [...]             //options for screens >= 1px width
      },
      {
        breakpoint: 768,  //second breakpoint
        [...]             //options for screens >= 768px width (wich is > first breakpoint)
      },
      {
        breakpoint: 992,  //third breakpoint
        [...]             //options for screens >= 992px width (wich is > second breakpoint)
      }
    ]
  });
```


## LICENSE

This project is licensed under the terms of the **MIT license**.

See [LICENSE.md](LICENSE.md) for detailed informations.