(function ($) {
  'use strict';

  //POLYFILLS
  //-----------------------------------------------
  //Request animation frame polyfill
  //http://www.javascriptkit.com/javatutors/requestanimationframe.shtml
  window.requestAnimationFrame = window.requestAnimationFrame || window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame || window.msRequestAnimationFrame || function (f) {
    return setTimeout(f, 1000 / 60);
  };
  //-----------------------------------------------

  //PRIVATE METHODS
  //-----------------------------------------------
  //get positions of element relative to viewport (offset from window bottom and top)
  function getPosition(elements, settings) {

  }

  //get element height
  function getElementHeight(element) {
    return element.height();
  }

  //check breakpoints
  function getBreakpoint(breakpointsArray) {
    var l = breakpointsArray.length - 1,
      viewportWidth = $(window).width();
    for (l; l >= 0; l = l - 1) {
      if (viewportWidth >= breakpointsArray[l]) {
        return l;
      }
    }
  }

  //set margins property based on reverseParallax value
  //inverting this properties inside the funtion parallax results in an inverted parallax effect
  function setMarginProperty(reverseParallax) {
    var margins = {};
    switch (reverseParallax) {
      case false:
        margins.first = 'margin-bottom';
        margins.second = 'margin-top';
        break;
      case true:
        margins.first = 'margin-top';
        margins.second = 'margin-bottom';
        break;
    }
    return margins;
  }

  //PLUGIN
  //-----------------------------------------------
  $.fn.parallax = function (userSettings) {
    this.each(function () {
      var settings = $.extend({}, $.fn.parallax.defaultSettings, userSettings),
        elements = {
          wrapper: $(this),
          img: $(this).children('img')
        },
        margins = {},
        currentSettings = {}, // reassign settings based on viewport width if responsive mode is enabled
        breakpointsArray = [],
        responsiveSettings = [],
        responsiveSettingsArray = [],
        currentBreakpoint = 0,
        elementHeight = 0,
        marginTop = 0, //placeholder for margin top to apply to image for parallax
        marginBottom = 0, //placeholder for margin bottom to apply to image for parallax
        elementTopOffset = 0, //placeholder for element offset from screen bottom
        elementBottomOffset = 0, //placeholder for element offset from screen top
        parallaxIndex = 0, // placeholder variable for the parallax index 
        viewportHeight = 0,
        i = 0,
        l = 0;

      //manage responsive settings
      if (settings.responsive) {
        //save responsive settings into array,
        responsiveSettings = settings.responsive;
        //delete settings.responsive;

        l = responsiveSettings.length;
        //i = 0;
        for (i; i < l; i = i + 1) {
          breakpointsArray.push(responsiveSettings[i].breakpoint);
          responsiveSettingsArray.push($.extend({}, settings, responsiveSettings[i]));
        }
      }

      //calculate height of image  
      //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
      elementHeight = getElementHeight(elements.img);
      viewportHeight = getElementHeight($(window));
      //end fix
      $(window).on('resize', function () {
        // Both ready and loaded
        elementHeight = getElementHeight(elements.img);
        viewportHeight = getElementHeight($(window));
      });

      //check wich settings to use based on viewport width (only for responsive mode)
      //reassign settings to use in function parallax
      //check parallax mode (normal or reverse)
      if (settings.responsive) {
        //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
        currentBreakpoint = getBreakpoint(breakpointsArray);
        currentSettings = responsiveSettingsArray[currentBreakpoint];
        margins = setMarginProperty(responsiveSettingsArray[currentBreakpoint].reverseParallax);
        //end fix
        $(window).on('resize ready', function () {
          // Both ready and loaded
          currentBreakpoint = getBreakpoint(breakpointsArray);
          currentSettings = responsiveSettingsArray[currentBreakpoint];
          margins = setMarginProperty(responsiveSettingsArray[currentBreakpoint].reverseParallax);
        });
      } else {
        currentSettings = settings;
        margins = setMarginProperty(settings.reverseParallax);
      }


      //animate image
      function parallax() {
        elementTopOffset = viewportHeight + $(window).scrollTop() - elements.wrapper.offset().top;
        elementBottomOffset = viewportHeight + elements.img.height() - settings.parallaxEmphasis;
        //animate element only if in viewport
        if (elementTopOffset > 0 && elementTopOffset < elementBottomOffset) {
          parallaxIndex = Math.round(elementHeight * settings.parallaxEmphasis / 100 * (elementTopOffset * 100 / elementBottomOffset) / 100);
          marginTop = -parallaxIndex;
          marginBottom = -(elementHeight * settings.parallaxEmphasis / 100 - parallaxIndex);
          elements.img.css(margins.first, marginTop).css(margins.second, marginBottom);
        }
      }

      //call parallax on scroll, load and resize
      //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
      window.requestAnimationFrame(parallax);
      //end fix
      $(window).on('scroll resize', function () {
        window.requestAnimationFrame(parallax);
      });
    });
  };


  $.fn.parallax.defaultSettings = {
    parallaxEmphasis: 50,
    reverseParallax: false,
    responsive: false
  };
}(jQuery));



$(document).ready(function () {
  'use strict';
  //Responsive
  $('#parallax-1, #parallax-2').parallax({
    parallaxEmphasis: 60,
    responsive: [
      {
        breakpoint: 0,
        parallaxEmphasis: 0
      },
      {
        breakpoint: 768,
        parallaxEmphasis: 40,
        reverseParallax: true
      },
      {
        breakpoint: 992,
        parallaxEmphasis: 50
      }
    ]
  });
});
