(function ($) {
  'use strict';

  //POLYFILLS
  //-----------------------------------------------
  //Request animation frame polyfill
  //http://www.javascriptkit.com/javatutors/requestanimationframe.shtml
  window.requestAnimationFrame = window.requestAnimationFrame || window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame || window.msRequestAnimationFrame || function (f) {
    return setTimeout(f, 1000 / 60);
  };
  //-----------------------------------------------

  //PRIVATE METHODS
  //-----------------------------------------------
  //get positions of element relative to viewport (offset from window bottom and top)
  function getPosition(elements, settings) {
    var height = $(window).height();
    return {
      position: height + $(window).scrollTop() - elements.wrapper.offset().top,
      positionMax: height + elements.img.height() - settings.parallaxEmphasis
    };
  }

  //get element height
  function getElementHeight(element) {
    return element.height();
  }

  //check if element is in viewport based on position
  function isInViewport(positions) {
    return positions.position > 0 && positions.position < positions.positionMax ? true : false;
  }

  //calculate index based on element's offset and emphasis
  function calculateParallaxMargins(elementHeight, positions, settings) {
    var index = Math.round(elementHeight * settings.parallaxEmphasis / 100 * (positions.position * 100 / positions.positionMax) / 100);
    return {
      marginTop: -index,
      marginBottom: -(elementHeight * settings.parallaxEmphasis / 100 - index)
    };
  }

  //check breakpoints
  function getBreakpoint(breakpointsArray) {
    var l = breakpointsArray.length - 1,
      viewportWidth = $(window).width();
    for (l; l >= 0; l = l - 1) {
      if (viewportWidth >= breakpointsArray[l]) {
        return l;
      }
    }
  }

  //set margins property based on reverseParallax value
  //inverting this properties inside the funtion parallax results in an inverted parallax effect
  function setMarginProperty(reverseParallax) {
    var margins = {};
    switch (reverseParallax) {
      case false:
        margins.first = 'margin-bottom';
        margins.second = 'margin-top';
        break;
      case true:
        margins.first = 'margin-top';
        margins.second = 'margin-bottom';
        break;
    }
    return margins;
  }

  //PLUGIN
  //-----------------------------------------------
  $.fn.parallax = function (userSettings) {
    var settings = $.extend({
        parallaxEmphasis: 50,
        reverseParallax: false,
        responsive: false
      }, userSettings),
      elements = {
        wrapper: $(this),
        img: $(this).children('img')
      },
      margins = {},
      currentSettings = {}, // reassign settings based on viewport width if responsive mode is enabled
      elementHeight = 0,
      breakpointsArray = [],
      responsiveSettings = [],
      responsiveSettingsArray = [],
      currentBreakpoint = 0,
      i = 0,
      l = 0;

    //manage responsive settings
    if (settings.responsive) {
      //save responsive settings into array,
      responsiveSettings = settings.responsive;
      //delete settings.responsive;

      l = responsiveSettings.length;
      //i = 0;
      for (i; i < l; i = i + 1) {
        breakpointsArray.push(responsiveSettings[i].breakpoint);
        responsiveSettingsArray.push($.extend({}, settings, responsiveSettings[i]));
      }
    }

    //animate image
    function parallax() {
      var positions = getPosition(elements, currentSettings),
        index = 0;
      //animate element only if in viewport
      if (isInViewport(positions)) {
        index = calculateParallaxMargins(elementHeight, positions, currentSettings);
        elements.img.css(margins.first, index.marginTop).css(margins.second, index.marginBottom);
      }
    }

    //calculate height of image  
    //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
    elementHeight = getElementHeight(elements.img);
    //end fix
    $(window).on('resize', function () {
      // Both ready and loaded
      elementHeight = getElementHeight(elements.img);
    });

    //check wich settings to use based on viewport width (only for responsive mode)
    //reassign settings to use in function parallax
    //check parallax mode (normal or reverse)

    if (settings.responsive) {
      //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
      currentBreakpoint = getBreakpoint(breakpointsArray);
      currentSettings = responsiveSettingsArray[currentBreakpoint];
      margins = setMarginProperty(responsiveSettingsArray[currentBreakpoint].reverseParallax);
      //end fix
      $(window).on('resize ready', function () {
        // Both ready and loaded
        currentBreakpoint = getBreakpoint(breakpointsArray);
        currentSettings = responsiveSettingsArray[currentBreakpoint];
        margins = setMarginProperty(responsiveSettingsArray[currentBreakpoint].reverseParallax);
      });
    } else {
      currentSettings = settings;
      margins = setMarginProperty(settings.reverseParallax);
    }

    //call parallax on scroll, load and resize
    //fix jQuery 3 problem (load event does not fires. https://github.com/jquery/jquery/issues/3194)
    window.requestAnimationFrame(parallax);
    //end fix
    $(window).on('scroll resize', function () {
      window.requestAnimationFrame(parallax);
    });
  };
}(jQuery));
